#!/usr/bin/python

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

from ansible.module_utils.basic import AnsibleModule

DOCUMENTATION = '''
---
module: ovh_nasha
short_description: Manage OVH NAS-HA ACL
description:
    - Manage OVH (French European hosting provider) NAS-HA ACL.
author: "Emmanuel Cohen (@atmaniak)"
notes:
    - Uses the python OVH Api U(https://github.com/ovh/python-ovh).
      You have to create an application (a key and secret) with a consummer
      key as described into U(https://eu.api.ovh.com/g934.first_step_with_api)
requirements:
    - ovh >=  0.4.8
options:
    name:
        required: true
        description:
            - Name of the zpoool to manage
    partition:
        required: true
        description:
            - The name of partition inside zpool
    ip:
        required: true
        description:
            - The IP address to register in ACL
    state:
        required: true
        default: present
        choices: ['present', 'absent']
        description:
            - Determines wether the record is to be created/modified or deleted
    endpoint:
        required: false
        default: ovh-eu
        description:
            - The endpoint to use (for instance ovh-eu)
    application_key:
        required: true
        description:
            - The applicationKey to use
    application_secret:
        required: true
        description:
            - The application secret to use
    consumer_key:
        required: true
        description:
            - The consumer key to use

'''

EXAMPLES = '''
# Allow 127.0.0.1 on partition mypartition on zpool-test
- ovh_nasha:
    name: zpool-test
    partition: mypartition
    ip: 127.0.0.1
    state: present
    application_key: yourkey
    application_secret: yoursecret
    consumer_key: yourconsumerkey
# Remove 127.0.0.1 on partition mypartition on zpool-test
- ovh_nasha:
    name: zpool-test
    partition: mypartition
    ip: 127.0.0.1
    state: absent
    endpoint: ovh-eu
    application_key: yourkey
    application_secret: yoursecret
    consumer_key: yourconsumerkey
'''

RETURN = '''
'''

try:
    import ovh
    import ovh.exceptions
    from ovh.exceptions import APIError
    HAS_OVH = True
except ImportError:
    HAS_OVH = False


def getOvhClient(ansibleModule):
    endpoint = ansibleModule.params.get('endpoint')
    application_key = ansibleModule.params.get('application_key')
    application_secret = ansibleModule.params.get('application_secret')
    consumer_key = ansibleModule.params.get('consumer_key')

    return ovh.Client(
        endpoint=endpoint,
        application_key=application_key,
        application_secret=application_secret,
        consumer_key=consumer_key
    )


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(required=True),
            partition=dict(required=True),
            ip=dict(required=True),
            state=dict(default='present', choices=['present', 'absent']),
            endpoint=dict(default='ovh-eu'),
            application_key=dict(required=True, no_log=True),
            application_secret=dict(required=True, no_log=True),
            consumer_key=dict(required=True, no_log=True),
        ),
        supports_check_mode=True
    )

    result = dict(
        changed=False
    )

    if not HAS_OVH:
        module.fail_json(msg='ovh-api python module is required to run this module ')

    # Get parameters
    name = module.params.get('name')
    partition = module.params.get('partition')
    ip = module.params.get('ip')
    state = module.params.get('state')

    # Connect to OVH API
    client = getOvhClient(module)

    # Check that the NAS HA and partition exists exists
    try:
        nasha = client.get('/dedicated/nasha/{}/partition'.format(name))
    except APIError as apiError:
        module.fail_json(
            msg='Unable to call OVH api for getting the this partition '
                'check application key, secret, consumerkey and parameters. '
                'Error returned by OVH api was : {0}'.format(apiError))

    if partition not in nasha:
        module.fail_json(msg='partition {0} does not exist on {1}'.format(partition, name))

    # Check that IP is not already allowed.
    try:
        allowed_ips = client.get('/dedicated/nasha/{}/partition/{}/access'.format(name, partition))
    except APIError as apiError:
        module.fail_json(
            msg='Unable to call OVH api for getting the properties '
                'of the ip, check application key, secret, consumerkey '
                'and parameters. Error returned by OVH api was : {0}'
            .format(apiError))

    # Allow IP if state present.
    if '{0}/32'.format(ip) not in allowed_ips and state == "present":
        if not module.check_mode:
            try:
                client.post('/dedicated/nasha/{}/partition/{}/access'.format(name, partition), ip=ip)
            except APIError as apiError:
                module.fail_json(
                    msg='Unable to call OVH api for getting the properties '
                        'of the ip, check application key, secret, consumerkey '
                        'and parameters. Error returned by OVH api was : {0}'
                    .format(apiError))

        result['changed'] = True

    # Remove IP if state absent.
    if '{0}/32'.format(ip) in allowed_ips and state == "absent":
        if not module.check_mode:
            try:
                client.delete('/dedicated/nasha/{}/partition/{}/access/{}'.format(name, partition, ip))
            except APIError as apiError:
                module.fail_json(
                    msg='Unable to call OVH api for getting the properties '
                        'of the ip, check application key, secret, consumerkey '
                        'and parameters. Error returned by OVH api was : {0}'
                    .format(apiError))
        result['changed'] = True

    module.exit_json(**result)


if __name__ == '__main__':
    main()
